from __future__ import absolute_import, unicode_literals

import unicodedata

import six
from unidecode import unidecode

PUNCS_WE_DONT_LIKE = "[],.()<>'/?;:\"&"


def kpick(args, options, default=None):
    for opt in options:
        if args.get(opt, None):
            return args[opt]
    return default


def remove_chars(inp, chars=PUNCS_WE_DONT_LIKE):
    for char in chars:
        inp = inp.replace(char, '')
    return inp


def asciify(inp):
    """ Nuke all the unicode from orbit. It's the only way to be sure.

    :param inp: (str)
    :return: string converted to pure, American ASCII
    """
    # TODO: be more diplomatic than an atomic bomb: convert international chars to ascii equivalents.
    # see http://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-in-a-python-unicode-string
    if inp:
        try:
            return inp.encode('ascii', 'ignore')
        except UnicodeDecodeError:
            return unicodedata.normalize('NFKD', inp.decode('utf-8')).encode('ascii', 'ignore')
    else:
        return ''


def squash_spaces(inp):
    """ Convert multiple ' ' chars to a single space.

    :param inp: (str)
    :return: same string with only one space where multiple spaces were.
    """
    return ' '.join(inp.split())


def parameterize(inp, sep='+'):
    """ Make strings suitable for submission to GET-based query service.

    Strips out the characters named in metapub.utils.PUNCS_WE_DONT_LIKE

    If inp is None, return empty string.

    :param inp: (str or None): input to be parameterized
    :param sep: (str): separator to use in place of spaces (default='+')
    :return: "parameterized" str
    """
    if inp is None:
        return ''

    inp = remove_chars(inp, PUNCS_WE_DONT_LIKE) 
    inp = squash_spaces(inp).replace(' ', sep)

    if six.PY2:
        return asciify(inp)
    else:
        return unidecode(inp)


def deparameterize(inp, sep='+'):
    """ Somewhat-undo parameterization in string. Replace separators (sep) with spaces.

    :param inp: (str)
    :param sep: (str) default: '+'
    :return: "deparameterized" string
    """
    return inp.replace(sep, ' ')


def remove_html_markup(inp):
    """ Remove html and xml tags from text.
    Preserves HTML entities like &amp;

    :param inp: (str)
    :return: string with HTML and XML markup removed.
    """
    tag = False
    quote = False
    out = ""

    for char in inp:
        if char == '<' and not quote:
            tag = True
        elif char == '>' and not quote:
            tag = False
        elif (char == '"' or char == "'") and tag:
            quote = not quote
        elif not tag:
            out = out + char
    return out


def lowercase_keys(dct):
    """ Takes an input dictionary, returns dictionary with all keys lowercased. """
    result = {}
    for key, value in list(dct.items()):
        result[key.lower()] = value
    return result



from __future__ import absolute_import, unicode_literals

import six

from lxml import etree

from .exceptions import MetaPubError


def parse_elink_response(xmlstr):
    '''return all Ids from an elink XML response'''
    if six.PY3 and type(xmlstr) == six.binary_type:
        xmlstr = xmlstr.decode()
    dom = etree.fromstring(xmlstr)
    ids = []
    if dom.find('LinkSet/LinkSetDb/LinkName').text:
        linkset = dom.find('LinkSet/LinkSetDb')
        for link in linkset.getchildren():
            if link.find('Id') is not None:
                ids.append(link.find('Id').text)
        return ids
    else:
        return None


class MetaPubObject(object):
    '''Base class for XML parsing objects (e.g. PubMedArticle)'''

    def __init__(self, xml, root=None, *args, **kwargs):
        '''Instantiate with "xml" as string or bytes containing valid XML.

        Supply name of root element (string) to set virtual top level. (optional).''' 
 
        if not xml:
            if xml == '':
                xml = 'empty'
            raise MetaPubError('Cannot build MetaPubObject; xml string was %s' % xml)
        self.xml = xml
        self.content = self.parse_xml(xml, root)

    @staticmethod
    def parse_xml(xml, root=None):
        '''Takes xml (str or bytes) and (optionally) a root element definition string.

        If root element defined, DOM object returned is rebased with this element as
        root.

        Args:
            xml (str or bytes)
            root (str): (optional) name of root element

        Returns:
            lxml document object.
        '''
        if isinstance(xml, str) or isinstance(xml, bytes):
            dom = etree.XML(xml)
        else:
            dom = etree.XML(xml)

        if root:
            return dom.find(root)
        else:
            return dom

    def _get(self, tag):
        '''Returns content of named XML element, or None if not found.'''
        elem = self.content.find(tag)
        if elem is not None:
            return elem.text
        return None

# singleton class used by the fetchers.
class Borg(object):
    '''singleton class backing cache engine objects.'''
    _shared_state = {}

    def __init__(self):
        self.__dict__ = self._shared_state



from __future__ import absolute_import, unicode_literals, print_function

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

from .pubmedarticle import PubMedArticle
from .pubmedfetcher import PubMedFetcher
from .medgenfetcher import MedGenFetcher
from .medgenconcept import MedGenConcept
from .crossref import CrossRef
from .findit import FindIt
from .dx_doi import DxDOI

__version__ = '0.4.0.1'




from __future__ import absolute_import, unicode_literals

JOURNAL_CANTDO_LIST = [
    'J Lancet',  # not to be confused with THE Lancet ("Lancet"), this one ended 1965, collection unlikely.
    'Nat Monspel Ser Bot',      # 1972-1973 http://www.unboundmedicine.com/medline/journal/Nat_Monspel_Ser_Bot
    'Nat Prod Lett',        # ended 2002.  http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Prod+Lett%22[TA]
    'Nat Hist',     # ended 1998.  http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Hist%22[TA]
    'Nat Mus',      # ended 1980. German.  http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Mus%22[TA]
    'Nat Syst',     # one article from 1982:  http://www.ncbi.nlm.nih.gov/pubmed/?term=11617791
    'Nat Resour Forum',   # ended 1997. http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Resour+Forum%22[TA]
    'Nat Law Forum',    # ended 1967. http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Law+Forum%22[TA]
    'Nat Resour J',     # ended 1982. http://www.ncbi.nlm.nih.gov/pubmed/?term=%22Nat+Resour+J%22[TA]
    'Zhonghua Yi Xue Yi Chuan Xue Za Zhi', # Chinese & DOI won't resolve. (eg. http://dx.doi.org/10.3760/cma.j.issn.1003-9406.2015.05.034)
]

# HELP NEEDED ESPECIALLY WITH CHINESE JOURNALS...!
# http://zhyxycx.periodicals.net.cn/default.html
